// ----------------------------------------------------------------------
//
// ParticleName.cc
// Author: Lynn Garren and Walter Brown
//
//  Create a map that gives a standard name for each pre-defined 
//  particle ID number.  This map is initialized if and only if 
//  the public functions are called. Because the map is static, 
//  the initialization happens only once.
//
//  The user NEVER calls ParticleNameInit()
//  We use a data table (struct Snames) so that compile time is not impacted.
//
//  public functions:
//     ParticleNameMap const &  getParticleNameMap()
//     std::string              ParticleName( const int pid )
//     void                     listHepPDTParticleNames( std::ostream & os  )
//
// ----------------------------------------------------------------------

#include <string>
#include <map>
#include <iostream>
#include <iomanip>	// width
#include <utility>	// make_pair

#include "CLHEP/HepPDT/ParticleName.hh"

namespace HepPDT {

typedef  std::map< int, std::string >  ParticleNameMap;

namespace {	// ParticleNameInit is private

ParticleNameMap const &  ParticleNameInit()
{

  static ParticleNameMap  m;

  static const struct {
      int pid;
      const char* pname;
  } SNames[] = {
      {          0, "" },
      {          1, "d" },
      {         -1, "d~" },
      {          2, "u" },
      {         -2, "u~" },
      {          3, "s" },
      {         -3, "s~" },
      {          4, "c" },
      {         -4, "c~" },
      {          5, "b" },
      {         -5, "b~" },
      {          6, "t" },
      {         -6, "t~" },
      {          7, "b'" },
      {         -7, "b'~" },
      {          8, "t'" },
      {         -8, "t'~" },
      {         11, "e-" },
      {        -11, "e+" },
      {         12, "nu_e" },
      {        -12, "nu_e~" },
      {         13, "mu-" },
      {        -13, "mu+" },
      {         14, "nu_mu" },
      {        -14, "nu_mu~" },
      {         15, "tau-" },
      {        -15, "tau+" },
      {         16, "nu_tau" },
      {        -16, "nu_tau~" },
      {         17, "tau'-" },
      {        -17, "tau'+" },
      {         18, "nu_tau'" },
      {        -18, "nu_tau'~" },
      {         21, "g" },
      {         22, "gamma" },
      {      10022, "virtual photon" },
      {      20022, "Cerenkov radiation" },
      {         23, "Z0" },
      {         24, "W+" },
      {        -24, "W-" },
      {         25, "H_10" },
      {         32, "Z_20" },
      {         33, "Z_30" },
      {         34, "W_2+" },
      {        -34, "W_2-" },
      {         35, "H_20" },
      {         36, "H_30" },
      {         37, "H+" },
      {        -37, "H-" },
      {         39, "G"  },
      {         41, "R0" },
      {        -41, "R~0" },
      {         42, "LQ_c" },
      {        -42, "LQ_c~" },
      {         51, "H_L0" },
      {         52, "H_1++" },
      {        -52, "H_1--" },
      {         53, "H_2+" },
      {        -53, "H_2-" },
      {         54, "H_2++" },
      {        -54, "H_2--" },
      {         55, "H_40" },
      {        -55, "H_40~" },
      {        110, "reggeon" },
      {        130, "KL0" },
      {        310, "KS0" },
      {        990, "pomeron" },
      {       9990, "odderon" },
      {    1000001, "L susy d" },
      {   -1000001, "L susy d~" },
      {    1000002, "L susy u" },
      {   -1000002, "L susy u~" },
      {    1000003, "L susy s" },
      {   -1000003, "L susy s~" },
      {    1000004, "L susy c" },
      {   -1000004, "L susy c~" },
      {    1000005, "L susy b" },
      {   -1000005, "L susy b~" },
      {    1000006, "L susy t" },
      {   -1000006, "L susy t~" },
      {    1000011, "L susy e-" },
      {   -1000011, "L susy e+" },
      {    1000012, "L susy nu_e" },
      {   -1000012, "L susy nu_e~" },
      {    1000013, "L susy mu-" },
      {   -1000013, "L susy mu+" },
      {    1000014, "L susy nu_mu" },
      {   -1000014, "L susy nu_mu~" },
      {    1000015, "L susy tau-" },
      {   -1000015, "L susy tau+" },
      {    1000016, "L susy nu_tau" },
      {   -1000016, "L susy nu_tau~" },
      {    1000021, "gluino" },
      {    1000022, "susy chi_10" },
      {    1000023, "susy chi_20" },
      {    1000024, "susy chi_1+" },
      {   -1000024, "susy chi_1-" },
      {    1000025, "susy chi_30" },
      {    1000035, "susy chi_40" },
      {    1000037, "susy chi_2+" },
      {   -1000037, "susy chi_2-" },
      {    1000039, "gravitino" },
      {    2000001, "R susy d" },
      {   -2000001, "R susy d~" },
      {    2000002, "R susy u" },
      {   -2000002, "R susy u~" },
      {    2000003, "R susy s" },
      {   -2000003, "R susy s~" },
      {    2000004, "R susy c" },
      {   -2000004, "R susy c~" },
      {    2000005, "R susy b" },
      {   -2000005, "R susy b~" },
      {    2000006, "R susy t" },
      {   -2000006, "R susy t~" },
      {    2000011, "R susy e-" },
      {   -2000011, "R susy e+" },
      {    2000012, "R susy nu_e" },
      {   -2000012, "R susy nu_e~" },
      {    2000013, "R susy mu-" },
      {   -2000013, "R susy mu+" },
      {    2000014, "R susy nu_mu" },
      {   -2000014, "R susy nu_mu~" },
      {    2000015, "R susy tau-" },
      {   -2000015, "R susy tau+" },
      {    2000016, "R susy nu_tau" },
      {   -2000016, "R susy nu_tau~" },
      {    3100021, "V8_tech" },
      {   -3100021, "V8_tech" },
      {    3000111, "pi_tech0" },
      {    3060111, "pi_tech_22_1" },
      {    3160111, "pi_tech_22_8" },
      {    3000113, "rho_tech0" },
      {    3130113, "rho_tech_11" },
      {    3140113, "rho_tech_12" },
      {    3150113, "rho_tech_21" },
      {    3160113, "rho_tech_22" },
      {    3000211, "pi_tech+" },
      {   -3000211, "pi_tech-" },
      {    3000213, "rho_tech+" },
      {   -3000213, "rho_tech-" },
      {    3000221, "pi'_tech" },
      {    3100221, "eta_tech" },
      {    3000223, "omega_tech" },
      {    4000001, "d*" },
      {   -4000001, "d*~" },
      {    4000002, "u*" },
      {   -4000002, "u*~" },
      {    4000011, "e*-" },
      {   -4000011, "e*+" },
      {    4000012, "nu*_e" },
      {   -4000012, "nu*_e~" },
      {    4000039, "G*" },
      {   -4000039, "G*~" },
      {    9900012, "nu_Re" },
      {   -9900012, "nu_Re~" },
      {    9900014, "nu_Rmu" },
      {   -9900014, "nu_Rmu~" },
      {    9900016, "nu_Rtau" },
      {   -9900016, "nu_Rtau~" },
      {    9900023, "Z_R0" },
      {   -9900023, "Z_R~0" },
      {    9900024, "W_R+" },
      {   -9900024, "W_R-" },
      {    9900061, "H_L++" },
      {   -9900061, "H_L--" },
      {    9900062, "H_R++" },
      {   -9900062, "H_R--" },
      {    9910113, "rho_diffr0" },
      {    9910211, "pi_diffr+" },
      {   -9910211, "pi_diffr-" },
      {    9910223, "omega_diffr" },
      {    9910333, "phi_diffr" },
      {    9910443, "psi_diffr" },
      {    9912112, "n_diffr0" },
      {   -9912112, "n_diffr~0" },
      {    9912212, "p_diffr+" },
      {   -9912212, "p_diffr~-" },
      {    9920022, "remnant photon" },
      {    9922212, "remnant nucleon" },
      {   -9922212, "remnant nucleon~" },
      {       1103, "dd_1" },
      {      -1103, "dd_1~" },
      {       2101, "ud_0" },
      {      -2101, "ud_0~" },
      {       2103, "ud_1" },
      {      -2103, "ud_1~" },
      {       2203, "uu_1" },
      {      -2203, "uu_1~" },
      {       3101, "sd_0" },
      {      -3101, "sd_0~" },
      {       3103, "sd_1" },
      {      -3103, "sd_1~" },
      {       3201, "su_0" },
      {      -3201, "su_0~" },
      {       3203, "su_1" },
      {      -3203, "su_1~" },
      {       3303, "ss_1" },
      {      -3303, "ss_1~" },
      {       4101, "cd_0" },
      {      -4101, "cd_0~" },
      {       4103, "cd_1" },
      {      -4103, "cd_1~" },
      {       4201, "cu_0" },
      {      -4201, "cu_0~" },
      {       4203, "cu_1" },
      {      -4203, "cu_1~" },
      {       4301, "cs_0" },
      {      -4301, "cs_0~" },
      {       4303, "cs_1" },
      {      -4303, "cs_1~" },
      {       4403, "cc_1" },
      {      -4403, "cc_1~" },
      {       5101, "bd_0" },
      {      -5101, "bd_0~" },
      {       5103, "bd_1" },
      {      -5103, "bd_1~" },
      {       5201, "bu_0" },
      {      -5201, "bu_0~" },
      {       5203, "bu_1" },
      {      -5203, "bu_1~" },
      {       5301, "bs_0" },
      {      -5301, "bs_0~" },
      {       5303, "bs_1" },
      {      -5303, "bs_1~" },
      {       5401, "bc_0" },
      {      -5401, "bc_0~" },
      {       5403, "bc_1" },
      {      -5403, "bc_1~" },
      {       5503, "bb_1" },
      {      -5503, "bb_1~" },
      {       6101, "td_0" },
      {      -6101, "td_0~" },
      {       6103, "td_1" },
      {      -6103, "td_1~" },
      {       6201, "tu_0" },
      {      -6201, "tu_0~" },
      {       6203, "tu_1" },
      {      -6203, "tu_1~" },
      {       6301, "ts_0" },
      {      -6301, "ts_0~" },
      {       6303, "ts_1" },
      {      -6303, "ts_1~" },
      {       6401, "tc_0" },
      {      -6401, "tc_0~" },
      {       6403, "tc_1" },
      {      -6403, "tc_1~" },
      {       6501, "tb_0" },
      {      -6501, "tb_0~" },
      {       6503, "tb_1" },
      {      -6503, "tb_1~" },
      {       6603, "tt_1" },
      {      -6603, "tt_1~" },
      {       7101, "b'd_0" },
      {      -7101, "b'd_0~" },
      {       7103, "b'd_1" },
      {      -7103, "b'd_1~" },
      {       7201, "b'u_0" },
      {      -7201, "b'u_0~" },
      {       7203, "b'u_1" },
      {      -7203, "b'u_1~" },
      {       7301, "b's_0" },
      {      -7301, "b's_0~" },
      {       7303, "b's_1" },
      {      -7303, "b's_1~" },
      {       7401, "b'c_0" },
      {      -7401, "b'c_0~" },
      {       7403, "b'c_1" },
      {      -7403, "b'c_1~" },
      {       7501, "b'b_0" },
      {      -7501, "b'b_0~" },
      {       7503, "b'b_1" },
      {      -7503, "b'b_1~" },
      {       7601, "b't_0" },
      {      -7601, "b't_0~" },
      {       7603, "b't_1" },
      {      -7603, "b't_1~" },
      {       7703, "b'b'_1" },
      {      -7703, "b'b'_1~" },
      {       8101, "t'd_0" },
      {      -8101, "t'd_0~" },
      {       8103, "t'd_1" },
      {      -8103, "t'd_1~" },
      {       8201, "t'u_0" },
      {      -8201, "t'u_0~" },
      {       8203, "t'u_1" },
      {      -8203, "t'u_1~" },
      {       8301, "t's_0" },
      {      -8301, "t's_0~" },
      {       8303, "t's_1" },
      {      -8303, "t's_1~" },
      {       8401, "t'c_0" },
      {      -8401, "t'c_0~" },
      {       8403, "t'c_1" },
      {      -8403, "t'c_1~" },
      {       8501, "t'b_0" },
      {      -8501, "t'b_0~" },
      {       8503, "t'b_1" },
      {      -8503, "t'b_1~" },
      {       8601, "t't_0" },
      {      -8601, "t't_0~" },
      {       8603, "t't_1" },
      {      -8603, "t't_1~" },
      {       8701, "t'b'_0" },
      {      -8701, "t'b'_0~" },
      {       8703, "t'b'_1" },
      {      -8703, "t'b'_1~" },
      {       8803, "t't'_1" },
      {      -8803, "t't'_1~" },
      {        111, "pi0" },
      {    9000111, "a_0(980)0" },
      {      10111, "a_0(1450)0" },
      {     100111, "pi(1300)0" },
      {    9010111, "pi(1800)0" },
      {        113, "rho(770)0" },
      {    9000113, "pi_1(1400)0" },
      {      10113, "b_1(1235)0" },
      {    9010113, "pi_1(1600)0" },
      {      20113, "a_1(1260)0" },
      {    9020113, "rho(1900)0" },
      {      30113, "rho(1700)0" },
      {    9030113, "rho(2150)0" },
      {     100113, "rho(1450)0" },
      {        115, "a_2(1320)0" },
      {    9000115, "pi_2(2100)0" },
      {      10115, "pi_2(1670)0" },
      {        117, "rho_3(1690)0" },
      {    9000117, "rho_3(1990)0" },
      {    9010117, "rho_3(2250)0" },
      {        119, "a_4(2040)0" },
      {        211, "pi+" },
      {       -211, "pi-" },
      {    9000211, "a_0(980)+" },
      {   -9000211, "a_0(980)-" },
      {      10211, "a_0(1450)+" },
      {     -10211, "a_0(1450)-" },
      {     100211, "pi(1300)+" },
      {    -100211, "pi(1300)-" },
      {    9010211, "pi(1800)+" },
      {   -9010211, "pi(1800)-" },
      {        213, "rho(770)+" },
      {       -213, "rho(770)-" },
      {    9000213, "pi_1(1400)+" },
      {   -9000213, "pi_1(1400)-" },
      {      10213, "b_1(1235)+" },
      {     -10213, "b_1(1235)-" },
      {    9010213, "pi_1(1600)+" },
      {   -9010213, "pi_1(1600)-" },
      {      20213, "a_1(1260)+" },
      {     -20213, "a_1(1260)-" },
      {    9020213, "rho(1900)+" },
      {   -9020213, "rho(1900)-" },
      {      30213, "rho(1700)+" },
      {     -30213, "rho(1700)-" },
      {    9030213, "rho(2150)+" },
      {   -9030213, "rho(2150)-" },
      {     100213, "rho(1450)+" },
      {    -100213, "rho(1450)-" },
      {        215, "a_2(1320)+" },
      {       -215, "a_2(1320)-" },
      {    9000215, "pi_2(2100)+" },
      {   -9000215, "pi_2(2100)-" },
      {      10215, "pi_2(1670)+" },
      {     -10215, "pi_2(1670)-" },
      {        217, "rho_3(1690)+" },
      {       -217, "rho_3(1690)-" },
      {    9000217, "rho_3(1990)+" },
      {   -9000217, "rho_3(1990)-" },
      {    9010217, "rho_3(2250)+" },
      {   -9010217, "rho_3(2250)-" },
      {        219, "a_4(2040)+" },
      {       -219, "a_4(2040)-" },
      {        221, "eta" },
      {    9000221, "f_0(600)" },
      {      10221, "f_0(1370)" },
      {    9010221, "f_0(980)" },
      {    9020221, "eta(1405)" },
      {    9030221, "f_0(1500)" },
      {    9040221, "f_0(2020)" },
      {    9050221, "f_0(2100)" },
      {    9060221, "f_0(2200)" },
      {    9070221, "f_0(2330)" },
      {     100221, "eta(1295)" },
      {        331, "eta'(958)" },
      {      10331, "f_0(1710)" },
      {     100331, "eta(1475)" },
      {        223, "omega(782)" },
      {    9000223, "f_1(1510)" },
      {      10223, "h_1(1170)" },
      {      20223, "f_1(1285)" },
      {      30223, "omega(1650)" },
      {     100223, "omega(1420)" },
      {        333, "phi(1020)" },
      {      10333, "h_1(1380)" },
      {      20333, "f_1(1420)" },
      {     100333, "phi(1680)" },
      {        225, "f_2(1270)" },
      {    9000225, "f_2(1430)" },
      {      10225, "eta_2(1645)" },
      {    9010225, "f_2(1565)" },
      {    9020225, "f_2(1640)" },
      {    9030225, "f_2(1810)" },
      {    9040225, "f_2(1910)" },
      {    9050225, "f_2(1950)" },
      {    9060225, "f_2(2010)" },
      {    9070225, "f_2(2150)" },
      {    9080225, "f_2(2300)" },
      {    9090225, "f_2(2340)" },
      {        335, "f'_2(1525)" },
      {      10335, "eta_2(1870)" },
      {        227, "omega_3(1670)" },
      {        337, "phi_3(1850)" },
      {        229, "f_4(2050)" },
      {    9000229, "f_J(2220)" },
      {    9010229, "f_4(2300)" },
      {        311, "K0" },
      {       -311, "K~0" },
      {    9000311, "K*_0(800)0" },
      {   -9000311, "K*_0(800)~0" },
      {      10311, "K*_0(1430)0" },
      {     -10311, "K*_0(1430)~0" },
      {     100311, "K(1460)0" },
      {    -100311, "K(1460)~0" },
      {    9010311, "K*_0(1950)0" },
      {   -9010311, "K*_0(1950)0" },
      {        321, "K+" },
      {       -321, "K-" },
      {    9000321, "K*_0(800)+" },
      {   -9000321, "K*_0(800)-" },
      {      10321, "K*_0(1430)+" },
      {     -10321, "K*_0(1430)-" },
      {     100321, "K(1460)+" },
      {    -100321, "K(1460)-" },
      {    9010321, "K*_0(1950)+" },
      {   -9010321, "K*_0(1950)-" },
      {        313, "K*(892)0" },
      {       -313, "K*(892)~0" },
      {      10313, "K_1(1270)0" },
      {     -10313, "K_1(1270)~0" },
      {      20313, "K_1(1400)0" },
      {     -20313, "K_1(1400)~0" },
      {      30313, "K*(1680)0" },
      {     -30313, "K*(1680)~0" },
      {     100313, "K*(1410)0" },
      {    -100313, "K*(1410)~0" },
      {        323, "K*(892)+" },
      {       -323, "K*(892)-" },
      {      10323, "K_1(1270)+" },
      {     -10323, "K_1(1270)-" },
      {      20323, "K_1(1400)+" },
      {     -20323, "K_1(1400)-" },
      {      30323, "K*(1680)+" },
      {     -30323, "K*(1680)-" },
      {     100323, "K*(1410)+" },
      {    -100323, "K*(1410)-" },
      {        315, "K*_2(1430)0" },
      {       -315, "K*_2(1430)~0" },
      {    9000315, "K_2(1580)0" },
      {   -9000315, "K_2(1580)~0" },
      {      10315, "K_2(1770)0" },
      {     -10315, "K_2(1770)~0" },
      {    9010315, "K_2(2250)0" },
      {   -9010315, "K_2(2250)~0" },
      {      20315, "K_2(1820)0" },
      {     -20315, "K_2(1820)~0" },
      {        325, "K*_2(1430)+" },
      {       -325, "K*_2(1430)-" },
      {    9000325, "K_2(1580)+" },
      {   -9000325, "K_2(1580)-" },
      {      10325, "K_2(1770)+" },
      {     -10325, "K_2(1770)-" },
      {    9010325, "K_2(2250)+" },
      {   -9010325, "K_2(2250)-" },
      {      20325, "K_2(1820)+" },
      {     -20325, "K_2(1820)-" },
      {     100325, "K_2(1980)+" },
      {    -100325, "K_2(1980)-" },
      {        317, "K*_3(1780)0" },
      {       -317, "K*_3(1780)~0" },
      {    9010317, "K_3(2320)0" },
      {   -9010317, "K_3(2320)~0" },
      {        327, "K*_3(1780)+" },
      {       -327, "K*_3(1780)-" },
      {    9010327, "K_3(2320)+" },
      {   -9010327, "K_3(2320)-" },
      {        319, "K*_4(2045)0" },
      {       -319, "K*_4(2045)~0" },
      {    9000319, "K_4(2500)0" },
      {   -9000319, "K_4(2500)~0" },
      {        329, "K*_4(2045)+" },
      {       -329, "K*_4(2045)-" },
      {    9000329, "K_4(2500)+" },
      {   -9000329, "K_4(2500)-" },
      {        411, "D+" },
      {       -411, "D-" },
      {      10411, "D*_0+" },
      {     -10411, "D*_0-" },
      {     100411, "D(2S)+" },
      {    -100411, "D(2S)-" },
      {        413, "D*(2010)+" },
      {       -413, "D*(2010)-" },
      {      10413, "D_1(2420)+" },
      {     -10413, "D_1(2420)-" },
      {      20413, "D_1(H)+" },
      {     -20413, "D_1(H)-" },
      {     100413, "D*(2S)+" },
      {    -100413, "D*(2S)+" },
      {        415, "D*_2(2460)+" },
      {       -415, "D*_2(2460)-" },
      {        421, "D0" },
      {       -421, "D~0" },
      {      10421, "D*_00" },
      {     -10421, "D*_0~0" },
      {     100421, "D(2S)0" },
      {    -100421, "D(2S)~0" },
      {        423, "D*(2007)0" },
      {       -423, "D*(2007)~0" },
      {      10423, "D_1(2420)0" },
      {     -10423, "D_1(2420)~0" },
      {      20423, "D_1(H)0" },
      {     -20423, "D_1(H)~0" },
      {     100423, "D*(2S)0" },
      {    -100423, "D*(2S)~0" },
      {        425, "D*_2(2460)0" },
      {       -425, "D*_2(2460)~0" },
      {        431, "D_s+" },
      {       -431, "D_s-" },
      {      10431, "D*_s0(2317)+" },
      {     -10431, "D*_s0(2317)-" },
      {        433, "D*_s+" },
      {       -433, "D*_s-" },
      {      10433, "D_s1(2536)+" },
      {     -10433, "D_s1(2536)-" },
      {      20433, "D_s1(2460)+" },
      {     -20433, "D_s1(2460)-" },
      {        435, "D*_s2(2573)+" },
      {       -435, "D*_s2(2573)-" },
      {        441, "eta_c(1S)" },
      {      10441, "chi_c0(1P)" },
      {     100441, "eta_c(2S)" },
      {        443, "J/psi(1S)" },
      {    9000443, "psi(4040)" },
      {      10443, "hc(1P)" },
      {    9010443, "psi(4160)" },
      {      20443, "chi_c1(1P)" },
      {    9020443, "psi(4415)" },
      {      30443, "psi(3770)" },
      {     100443, "psi(2S)" },
      {        445, "chi_c2(1P)" },
      {    9000445, "psi(3836)" },
      {        511, "B0" },
      {       -511, "B~0" },
      {      10511, "B*_00" },
      {     -10511, "B*_0~0" },
      {        513, "B*0" },
      {       -513, "B*~0" },
      {      10513, "B_1(L)0" },
      {     -10513, "B_1(L)~0" },
      {      20513, "B_1(H)0" },
      {     -20513, "B_1(H)~0" },
      {        515, "B*_20" },
      {       -515, "B*_2~0" },
      {        521, "B+" },
      {       -521, "B-" },
      {      10521, "B*_0+" },
      {     -10521, "B*_0-" },
      {        523, "B*+" },
      {       -523, "B*-" },
      {      10523, "B_1(L)+" },
      {     -10523, "B_1(L)-" },
      {      20523, "B_1(H)+" },
      {     -20523, "B_1(H)-" },
      {        525, "B*_2+" },
      {       -525, "B*_2-" },
      {        531, "B_s0" },
      {       -531, "B_s~0" },
      {      10531, "B*_s00" },
      {     -10531, "B*_s0~0" },
      {        533, "B*_s0" },
      {       -533, "B*_s~0" },
      {      10533, "B_s1(L)0" },
      {     -10533, "B_s1(L)~0" },
      {      20533, "B_s1(H)0" },
      {     -20533, "B_s1(H)~0" },
      {        535, "B*_s20" },
      {       -535, "B*_s2~0" },
      {        541, "B_c+" },
      {       -541, "B_c-" },
      {      10541, "B*_c0+" },
      {     -10541, "B*_c0-" },
      {        543, "B*_c+" },
      {       -543, "B*_c-" },
      {      10543, "B_c1(L)+" },
      {     -10543, "B_c1(L)-" },
      {      20543, "B_c1(H)+" },
      {     -20543, "B_c1(H)-" },
      {        545, "B*_c2+" },
      {       -545, "B*_c2-" },
      {        551, "eta_b(1S)" },
      {      10551, "chi_b0(1P)" },
      {     100551, "eta_b(2S)" },
      {     110551, "chi_b0(2P)" },
      {     200551, "eta_b(3S)" },
      {     210551, "chi_b0(3P)" },
      {        553, "upsilon(1S)" },
      {    9000553, "upsilon(10860)" },
      {      10553, "h_b(1P)" },
      {    9010553, "upsilon(11020)" },
      {      20553, "chi_b1(1P)" },
      {    9020553, "upsilon(7S)" },
      {      30553, "upsilon_1(1D)" },
      {     100553, "upsilon(2S)" },
      {     110553, "h_b(2P)" },
      {     120553, "chi_b1(2P)" },
      {     130553, "upsilon_1(2D)" },
      {     200553, "upsilon(3S)" },
      {     210553, "h_b(3P)" },
      {     220553, "chi_b1(3P)" },
      {     300553, "upsilon(4S)" },
      {        555, "chi_b2(1P)" },
      {      10555, "eta_b2(1D)" },
      {      20555, "upsilon_2(1D)" },
      {     100555, "chi_b2(2P)" },
      {     110555, "eta_b2(2D)" },
      {     120555, "upsilon_2(2D)" },
      {     200555, "chi_b2(3P)" },
      {        557, "upsilon_3(1D)" },
      {     100557, "upsilon_3(2D)" },
      {        611, "T+" },
      {       -611, "T-" },
      {        613, "T*+" },
      {       -613, "T*-" },
      {        621, "T0" },
      {       -621, "T~0" },
      {        623, "T*0" },
      {       -623, "T*~0" },
      {        631, "T_s+" },
      {       -631, "T_s-" },
      {        633, "T*_s+" },
      {       -633, "T*_s-" },
      {        641, "T_c0" },
      {       -641, "T_c~0" },
      {        643, "T*_c0" },
      {       -643, "T*_c~0" },
      {        651, "T_b+" },
      {       -651, "T_b-" },
      {        653, "T*_b+" },
      {       -653, "T*_b-" },
      {        661, "eta_t" },
      {        663, "theta" },
      {        711, "L0" },
      {       -711, "L~0" },
      {        713, "L*0" },
      {       -713, "L*~0" },
      {        721, "L-" },
      {       -721, "L+" },
      {        723, "L*-" },
      {       -723, "L*+" },
      {        731, "L_s0" },
      {       -731, "L_s~0" },
      {        733, "L*_s0" },
      {       -733, "L*_s~0" },
      {        741, "L_c-" },
      {       -741, "L_c+" },
      {        743, "L*_c-" },
      {       -743, "L*_c+" },
      {        751, "L_b0" },
      {       -751, "L_b~0" },
      {        753, "L*_b0" },
      {       -753, "L*_b~0" },
      {        761, "L_t-" },
      {       -761, "L_t+" },
      {        763, "L*_t-" },
      {       -763, "L*_t+" },
      {        771, "eta_l" },
      {        773, "theta_l" },
      {        811, "H+" },
      {       -811, "H-" },
      {        813, "H*+" },
      {       -813, "H*-" },
      {        821, "H0" },
      {       -821, "H~0" },
      {        823, "H*0" },
      {       -823, "H*~0" },
      {        831, "H_s+" },
      {       -831, "H_s-" },
      {        833, "H*_s+" },
      {       -833, "H*_s-" },
      {        841, "H_c0" },
      {       -841, "H_c~0" },
      {        843, "H*_c0" },
      {       -843, "H*_c~0" },
      {        851, "H_b+" },
      {       -851, "H_b-" },
      {        853, "H*_b+" },
      {       -853, "H*_b-" },
      {        861, "H_t0" },
      {       -861, "H_t~0" },
      {        863, "H*_t0" },
      {       -863, "H*_t~0" },
      {        871, "H_l+" },
      {       -871, "H_l-" },
      {        873, "H*_l+" },
      {       -873, "H*_l-" },
      {        881, "eta_h" },
      {        883, "theta_H" },
      {       1114, "Delta-" },
      {      -1114, "Delta~+" },
      {       2112, "n0" },
      {      -2112, "n~0" },
      {       2114, "Delta0" },
      {      -2114, "Delta~0" },
      {       2212, "p+" },
      {      -2212, "p~-" },
      {       2214, "Delta+" },
      {      -2214, "Delta~-" },
      {       2224, "Delta++" },
      {      -2224, "Delta~--" },
      {       3112, "Sigma-" },
      {      -3112, "Sigma~+" },
      {       3114, "Sigma*-" },
      {      -3114, "Sigma*~+" },
      {       3122, "Lambda0" },
      {      -3122, "Lambda~0" },
      {       3212, "Sigma0" },
      {      -3212, "Sigma~0" },
      {       3214, "Sigma*0" },
      {      -3214, "Sigma*~0" },
      {       3222, "Sigma+" },
      {      -3222, "Sigma~-" },
      {       3224, "Sigma*+" },
      {      -3224, "Sigma*~-" },
      {       3312, "Xi-" },
      {      -3312, "Xi~+" },
      {       3314, "Xi*-" },
      {      -3314, "Xi*~+" },
      {       3322, "Xi0" },
      {      -3322, "Xi~0" },
      {       3324, "Xi*0" },
      {      -3324, "Xi*~0" },
      {       3334, "Omega-" },
      {      -3334, "Omega~+" },
      {       4112, "Sigma_c0" },
      {      -4112, "Sigma_c~0" },
      {       4114, "Sigma*_c0" },
      {      -4114, "Sigma*_c~0" },
      {       4122, "Lambda_c+" },
      {      -4122, "Lambda_c~-" },
      {       4132, "Xi_c0" },
      {      -4132, "Xi_c~0" },
      {       4212, "Sigma_c+" },
      {      -4212, "Sigma_c~-" },
      {       4214, "Sigma*_c+" },
      {      -4214, "Sigma*_c~-" },
      {       4222, "Sigma_c++" },
      {      -4222, "Sigma_c~--" },
      {       4224, "Sigma*_c++" },
      {      -4224, "Sigma*_c~--" },
      {       4232, "Xi_c+" },
      {      -4232, "Xi_c~-" },
      {       4312, "Xi'_c0" },
      {      -4312, "Xi'_c~0" },
      {       4314, "Xi*_c0" },
      {      -4314, "Xi*_c~0" },
      {       4322, "Xi'_c+" },
      {      -4322, "Xi'_c~-" },
      {       4324, "Xi*_c+" },
      {      -4324, "Xi*_c~-" },
      {       4332, "Omega_c0" },
      {      -4332, "Omega_c~0" },
      {       4334, "Omega*_c0" },
      {      -4334, "Omega*_c~0" },
      {       4412, "Xi_cc+" },
      {      -4412, "Xi_cc~-" },
      {       4414, "Xi*_cc+" },
      {      -4414, "Xi*_cc~-" },
      {       4422, "Xi_cc++" },
      {      -4422, "Xi_cc~--" },
      {       4424, "Xi*_cc++" },
      {      -4424, "Xi*_cc~--" },
      {       4432, "Omega_cc+" },
      {      -4432, "Omega_cc~-" },
      {       4434, "Omega*_cc+" },
      {      -4434, "Omega*_cc~-" },
      {       4444, "Omega*_ccc++" },
      {      -4444, "Omega*_ccc~--" },
      {       5112, "Sigma_b-" },
      {      -5112, "Sigma_b~+" },
      {       5114, "Sigma*_b-" },
      {      -5114, "Sigma*_b~+" },
      {       5122, "Lambda_b0" },
      {      -5122, "Lambda_b~0" },
      {       5132, "Xi_b-" },
      {      -5132, "Xi_b~+" },
      {       5142, "Xi_bc0" },
      {      -5142, "Xi_bc~0" },
      {       5212, "Sigma_b0" },
      {      -5212, "Sigma_b~0" },
      {       5214, "Sigma*_b0" },
      {      -5214, "Sigma*_b~0" },
      {       5222, "Sigma_b+" },
      {      -5222, "Sigma_b~-" },
      {       5224, "Sigma*_b+" },
      {      -5224, "Sigma*_b~-" },
      {       5232, "Xi_b0" },
      {      -5232, "Xi_b~0" },
      {       5242, "Xi_bc+" },
      {      -5242, "Xi_bc~-" },
      {       5312, "Xi'_b-" },
      {      -5312, "Xi'_b~+" },
      {       5314, "Xi*_b-" },
      {      -5314, "Xi*_b~+" },
      {       5322, "Xi'_b0" },
      {      -5322, "Xi'_b~0" },
      {       5324, "Xi*_b0" },
      {      -5324, "Xi*_b~0" },
      {       5332, "Omega_b-" },
      {      -5332, "Omega_b~+" },
      {       5334, "Omega*_b-" },
      {      -5334, "Omega*_b~+" },
      {       5342, "Omega_bc0" },
      {      -5342, "Omega_bc~0" },
      {       5412, "Xi'_bc0" },
      {      -5412, "Xi'_bc~0" },
      {       5414, "Xi*_bc0" },
      {      -5414, "Xi*_bc~0" },
      {       5422, "Xi'_bc+" },
      {      -5422, "Xi'_bc~-" },
      {       5424, "Xi*_bc+" },
      {      -5424, "Xi*_bc~-" },
      {       5432, "Omega'_bc0" },
      {      -5432, "Omega'_bc~0" },
      {       5434, "Omega*_bc0" },
      {      -5434, "Omega*_bc~0" },
      {       5442, "Omega_bcc+" },
      {      -5442, "Omega_bcc~-" },
      {       5444, "Omega*_bcc+" },
      {      -5444, "Omega*_bcc~-" },
      {       5512, "Xi_bb-" },
      {      -5512, "Xi_bb~+" },
      {       5514, "Xi*_bb-" },
      {      -5514, "Xi*_bb~+" },
      {       5522, "Xi_bb0" },
      {      -5522, "Xi_bb~0" },
      {       5524, "Xi*_bb0" },
      {      -5524, "Xi*_bb~0" },
      {       5532, "Omega_bb-" },
      {      -5532, "Omega_bb~+" },
      {       5534, "Omega*_bb-" },
      {      -5534, "Omega*_bb~+" },
      {       5542, "Omega_bbc0" },
      {      -5542, "Omega_bbc~0" },
      {       5544, "Omega*_bbc0" },
      {      -5544, "Omega*_bbc~0" },
      {       5554, "Omega*_bbb-" },
      {      -5554, "Omega*_bbb~+" },
      {       6112, "Sigma_t0" },
      {      -6112, "Sigma_t~0" },
      {       6114, "Sigma*_t0" },
      {      -6114, "Sigma*_t~0" },
      {       6122, "Lambda_t+" },
      {      -6122, "Lambda_t~-" },
      {       6132, "Xi_t0" },
      {      -6132, "Xi_t~0" },
      {       6142, "Xi_tc+" },
      {      -6142, "Xi_tc~-" },
      {       6152, "Xi_tb0" },
      {      -6152, "Xi_tb~0" },
      {       6212, "Sigma_t+" },
      {      -6212, "Sigma_t~-" },
      {       6214, "Sigma*_t+" },
      {      -6214, "Sigma*_t~-" },
      {       6222, "Sigma_t++" },
      {      -6222, "Sigma_t~--" },
      {       6224, "Sigma*_t++" },
      {      -6224, "Sigma*_t~--" },
      {       6232, "Xi_t+" },
      {      -6232, "Xi_t~-" },
      {       6242, "Xi_tc++" },
      {      -6242, "Xi_tc~--" },
      {       6252, "Xi_tb+" },
      {      -6252, "Xi_tb~-" },
      {       6312, "Xi'_t0" },
      {      -6312, "Xi'_t~0" },
      {       6314, "Xi*_t0" },
      {      -6314, "Xi*_t~0" },
      {       6322, "Xi'_t+" },
      {      -6322, "Xi'_t~-" },
      {       6324, "Xi*_t+" },
      {      -6324, "Xi*_t~-" },
      {       6332, "Omega_t0" },
      {      -6332, "Omega_t~0" },
      {       6334, "Omega*_t0" },
      {      -6334, "Omega*_t~0" },
      {       6342, "Omega_tc+" },
      {      -6342, "Omega_tc~-" },
      {       6352, "Omega_tb0" },
      {      -6352, "Omega_tb~0" },
      {       6412, "Xi'_tc+" },
      {      -6412, "Xi'_tc~-" },
      {       6414, "Xi*_tc+" },
      {      -6414, "Xi*_tc~-" },
      {       6422, "Xi'_tc++" },
      {      -6422, "Xi'_tc~--" },
      {       6424, "Xi*_tc++" },
      {      -6424, "Xi*_tc~--" },
      {       6432, "Omega'_tc+" },
      {      -6432, "Omega'_tc~-" },
      {       6434, "Omega*_tc+" },
      {      -6434, "Omega*_tc~-" },
      {       6442, "Omega_tcc++" },
      {      -6442, "Omega_tcc~--" },
      {       6444, "Omega*_tcc++" },
      {      -6444, "Omega*_tcc~--" },
      {       6452, "Omega_tbc+" },
      {      -6452, "Omega_tbc~-" },
      {       6512, "Xi'_tb0" },
      {      -6512, "Xi'_tb~0" },
      {       6514, "Xi*_tb0" },
      {      -6514, "Xi*_tb~0" },
      {       6522, "Xi'_tb+" },
      {      -6522, "Xi'_tb~-" },
      {       6524, "Xi*_tb+" },
      {      -6524, "Xi*_tb~-" },
      {       6532, "Omega'_tb0" },
      {      -6532, "Omega'_tb~0" },
      {       6534, "Omega*_tb0" },
      {      -6534, "Omega*_tb~0" },
      {       6542, "Omega'_tbc+" },
      {      -6542, "Omega'_tbc~-" },
      {       6544, "Omega*_tbc+" },
      {      -6544, "Omega*_tbc~-" },
      {       6552, "Omega_tbb0" },
      {      -6552, "Omega_tbb~0" },
      {       6554, "Omega*_tbb0" },
      {      -6554, "Omega*_tbb~0" },
      {       6612, "Xi_tt+" },
      {      -6612, "Xi_tt~-" },
      {       6614, "Xi*_tt+" },
      {      -6614, "Xi*_tt~-" },
      {       6622, "Xi_tt++" },
      {      -6622, "Xi_tt~--" },
      {       6624, "Xi*_tt++" },
      {      -6624, "Xi*_tt~--" },
      {       6632, "Omega_tt+" },
      {      -6632, "Omega_tt~-" },
      {       6634, "Omega*_tt+" },
      {      -6634, "Omega*_tt~-" },
      {       6642, "Omega_ttc++" },
      {      -6642, "Omega_ttc~--" },
      {       6644, "Omega*_ttc++" },
      {      -6644, "Omega*_ttc~--" },
      {       6652, "Omega_ttb+" },
      {      -6652, "Omega_ttb~-" },
      {       6654, "Omega*_ttb+" },
      {      -6654, "Omega*_ttb~-" },
      {       6664, "Omega*_ttt++" },
      {      -6664, "Omega*_ttt~--" },
      {       7112, "Sigma_b'-" },
      {      -7112, "Sigma_b'~+" },
      {       7114, "Sigma*_b'-" },
      {      -7114, "Sigma*_b'~+" },
      {       7122, "Lambda_b'0" },
      {      -7122, "Lambda_b'~0" },
      {       7132, "Xi_b'-" },
      {      -7132, "Xi_b'~+" },
      {       7142, "Xi_b'c0" },
      {      -7142, "Xi_b'c~0" },
      {       7152, "Xi_b'b-" },
      {      -7152, "Xi_b'b~+" },
      {       7162, "Xi_b't0" },
      {      -7162, "Xi_b't~0" },
      {       7212, "Sigma_b'0" },
      {      -7212, "Sigma_b'~0" },
      {       7214, "Sigma*_b'0" },
      {      -7214, "Sigma*_b'~0" },
      {       7222, "Sigma_b'+" },
      {      -7222, "Sigma_b'~-" },
      {       7224, "Sigma*_b'+" },
      {      -7224, "Sigma*_b'~-" },
      {       7232, "Xi_b'0" },
      {      -7232, "Xi_b'~0" },
      {       7242, "Xi_b'c+" },
      {      -7242, "Xi_b'c~-" },
      {       7252, "Xi_b'b0" },
      {      -7252, "Xi_b'b~0" },
      {       7262, "Xi_b't+" },
      {      -7262, "Xi_b't~-" },
      {       7312, "Xi'_b'-" },
      {      -7312, "Xi'_b'~+" },
      {       7314, "Xi*_b'-" },
      {      -7314, "Xi*_b'~+" },
      {       7322, "Xi'_b'0" },
      {      -7322, "Xi'_b'~0" },
      {       7324, "Xi*_b'0" },
      {      -7324, "Xi*_b'~0" },
      {       7332, "Omega'_b'-" },
      {      -7332, "Omega'_b'~+" },
      {       7334, "Omega*_b'-" },
      {      -7334, "Omega*_b'~+" },
      {       7342, "Omega_b'c0" },
      {      -7342, "Omega_b'c~0" },
      {       7352, "Omega_b'b-" },
      {      -7352, "Omega_b'b~+" },
      {       7362, "Omega_b't0" },
      {      -7362, "Omega_b't~0" },
      {       7412, "Xi'_b'c0" },
      {      -7412, "Xi'_b'c~0" },
      {       7414, "Xi*_b'c0" },
      {      -7414, "Xi*_b'c~0" },
      {       7422, "Xi'_b'c+" },
      {      -7422, "Xi'_b'c~-" },
      {       7424, "Xi*_b'c+" },
      {      -7424, "Xi*_b'c~-" },
      {       7432, "Omega'_b'c0" },
      {      -7432, "Omega'_b'c~0" },
      {       7434, "Omega*_b'c0" },
      {      -7434, "Omega*_b'c~0" },
      {       7442, "Omega'_b'cc+" },
      {      -7442, "Omega'_b'cc~-" },
      {       7444, "Omega*_b'cc+" },
      {      -7444, "Omega*_b'cc~-" },
      {       7452, "Omega_b'bc0" },
      {      -7452, "Omega_b'bc~0" },
      {       7462, "Omega_b'tc+" },
      {      -7462, "Omega_b'tc~-" },
      {       7512, "Xi'_b'b-" },
      {      -7512, "Xi'_b'b~+" },
      {       7514, "Xi*_b'b-" },
      {      -7514, "Xi*_b'b~+" },
      {       7522, "Xi'_b'b0" },
      {      -7522, "Xi'_b'b~0" },
      {       7524, "Xi*_b'b0" },
      {      -7524, "Xi*_b'b~0" },
      {       7532, "Omega'_b'b-" },
      {      -7532, "Omega'_b'b~+" },
      {       7534, "Omega*_b'b-" },
      {      -7534, "Omega*_b'b~+" },
      {       7542, "Omega'_b'bc0" },
      {      -7542, "Omega'_b'bc~0" },
      {       7544, "Omega*_b'bc0" },
      {      -7544, "Omega*_b'bc~0" },
      {       7552, "Omega'_b'bb-" },
      {      -7552, "Omega'_b'bb~+" },
      {       7554, "Omega*_b'bb-" },
      {      -7554, "Omega*_b'bb~+" },
      {       7562, "Omega_b'tb0" },
      {      -7562, "Omega_b'tb~0" },
      {       7612, "Xi'_b't0" },
      {      -7612, "Xi'_b't~0" },
      {       7614, "Xi*_b't0" },
      {      -7614, "Xi*_b't~0" },
      {       7622, "Xi'_b't+" },
      {      -7622, "Xi'_b't~-" },
      {       7624, "Xi*_b't+" },
      {      -7624, "Xi*_b't~-" },
      {       7632, "Omega'_b't0" },
      {      -7632, "Omega'_b't~0" },
      {       7634, "Omega*_b't0" },
      {      -7634, "Omega*_b't~0" },
      {       7642, "Omega'_b'tc+" },
      {      -7642, "Omega'_b'tc~-" },
      {       7644, "Omega*_b'tc+" },
      {      -7644, "Omega*_b'tc~-" },
      {       7652, "Omega'_b'tb0" },
      {      -7652, "Omega'_b'tb~0" },
      {       7654, "Omega*_b'tb0" },
      {      -7654, "Omega*_b'tb~0" },
      {       7662, "Omega'_b'tt+" },
      {      -7662, "Omega'_b'tt~-" },
      {       7664, "Omega*_b'tt+" },
      {      -7664, "Omega*_b'tt~-" },
      {       7712, "Xi'_b'b'-" },
      {      -7712, "Xi'_b'b'~+" },
      {       7714, "Xi*_b'b'-" },
      {      -7714, "Xi*_b'b'~+" },
      {       7722, "Xi'_b'b'0" },
      {      -7722, "Xi'_b'b'~0" },
      {       7724, "Xi*_b'b'0" },
      {      -7724, "Xi*_b'b'~0" },
      {       7732, "Omega'_b'b'-" },
      {      -7732, "Omega'_b'b'~+" },
      {       7734, "Omega*_b'b'-" },
      {      -7734, "Omega*_b'b'~+" },
      {       7742, "Omega'_b'b'c0" },
      {      -7742, "Omega'_b'b'c~0" },
      {       7744, "Omega*_b'b'c0" },
      {      -7744, "Omega*_b'b'c~0" },
      {       7752, "Omega'_b'b'b-" },
      {      -7752, "Omega'_b'b'b~+" },
      {       7754, "Omega*_b'b'b-" },
      {      -7754, "Omega*_b'b'b~+" },
      {       7762, "Omega'_b'b't0" },
      {      -7762, "Omega'_b'b't~0" },
      {       7764, "Omega*_b'b't0" },
      {      -7764, "Omega*_b'b't~0" },
      {       7774, "Omega*_b'b'b'-" },
      {      -7774, "Omega*_b'b'b'~+" },
      {       8112, "Sigma_t'0" },
      {      -8112, "Sigma_t'~0" },
      {       8114, "Sigma*_t'0" },
      {      -8114, "Sigma*_t'~0" },
      {       8122, "Lambda_t'+" },
      {      -8122, "Lambda_t'~-" },
      {       8132, "Xi_t'0" },
      {      -8132, "Xi_t'~0" },
      {       8142, "Xi_t'c+" },
      {      -8142, "Xi_t'c~-" },
      {       8152, "Xi_t'b0" },
      {      -8152, "Xi_t'b~0" },
      {       8162, "Xi_t't+" },
      {      -8162, "Xi_t't~-" },
      {       8172, "Xi_t'b'0" },
      {      -8172, "Xi_t'b'~0" },
      {       8212, "Sigma_t'+" },
      {      -8212, "Sigma_t'~-" },
      {       8214, "Sigma*_t'+" },
      {      -8214, "Sigma*_t'~-" },
      {       8222, "Sigma_t'++" },
      {      -8222, "Sigma_t'~--" },
      {       8224, "Sigma*_t'++" },
      {      -8224, "Sigma*_t'~--" },
      {       8232, "Xi_t'+" },
      {      -8232, "Xi_t'~-" },
      {       8242, "Xi_t'c++" },
      {      -8242, "Xi_t'c~--" },
      {       8252, "Xi_t'b+" },
      {      -8252, "Xi_t'b~-" },
      {       8262, "Xi_t't++" },
      {      -8262, "Xi_t't~--" },
      {       8272, "Xi_t'b'+" },
      {      -8272, "Xi_t'b'~-" },
      {       8312, "Xi'_t'0" },
      {      -8312, "Xi'_t'~0" },
      {       8314, "Xi*_t'0" },
      {      -8314, "Xi*_t'~0" },
      {       8322, "Xi'_t'+" },
      {      -8322, "Xi'_t'~-" },
      {       8324, "Xi*_t'+" },
      {      -8324, "Xi*_t'~-" },
      {       8332, "Omega'_t'0" },
      {      -8332, "Omega'_t'~0" },
      {       8334, "Omega*_t'0" },
      {      -8334, "Omega*_t'~0" },
      {       8342, "Omega_t'c+" },
      {      -8342, "Omega_t'c~-" },
      {       8352, "Omega_t'b0" },
      {      -8352, "Omega_t'b~0" },
      {       8362, "Omega_t't+" },
      {      -8362, "Omega_t't~-" },
      {       8372, "Omega_t'b'0" },
      {      -8372, "Omega_t'b'~0" },
      {       8412, "Xi'_t'c+" },
      {      -8412, "Xi'_t'c~-" },
      {       8414, "Xi*_t'c+" },
      {      -8414, "Xi*_t'c~-" },
      {       8422, "Xi'_t'c++" },
      {      -8422, "Xi'_t'c~--" },
      {       8424, "Xi*_t'c++" },
      {      -8424, "Xi*_t'c~--" },
      {       8432, "Omega'_t'c+" },
      {      -8432, "Omega'_t'c~-" },
      {       8434, "Omega*_t'c+" },
      {      -8434, "Omega*_t'c~-" },
      {       8442, "Omega'_t'cc++" },
      {      -8442, "Omega'_t'cc~--" },
      {       8444, "Omega*_t'cc++" },
      {      -8444, "Omega*_t'cc~--" },
      {       8452, "Omega_t'bc+" },
      {      -8452, "Omega_t'bc~-" },
      {       8462, "Omega_t'tc++" },
      {      -8462, "Omega_t'tc~--" },
      {       8472, "Omega_t'b'c +" },
      {      -8472, "Omega_t'b'c ~-" },
      {       8512, "Xi'_t'b0" },
      {      -8512, "Xi'_t'b~0" },
      {       8514, "Xi*_t'b0" },
      {      -8514, "Xi*_t'b~0" },
      {       8522, "Xi'_t'b+" },
      {      -8522, "Xi'_t'b~-" },
      {       8524, "Xi*_t'b+" },
      {      -8524, "Xi*_t'b~-" },
      {       8532, "Omega'_t'b0" },
      {      -8532, "Omega'_t'b~0" },
      {       8534, "Omega*_t'b0" },
      {      -8534, "Omega*_t'b~0" },
      {       8542, "Omega'_t'bc+" },
      {      -8542, "Omega'_t'bc~-" },
      {       8544, "Omega*_t'bc+" },
      {      -8544, "Omega*_t'bc~-" },
      {       8552, "Omega'_t'bb0" },
      {      -8552, "Omega'_t'bb~0" },
      {       8554, "Omega*_t'bb0" },
      {      -8554, "Omega*_t'bb~0" },
      {       8562, "Omega_t'tb+" },
      {      -8562, "Omega_t'tb~-" },
      {       8572, "Omega_t'b'b 0" },
      {      -8572, "Omega_t'b'b ~0" },
      {       8612, "Xi'_t't+" },
      {      -8612, "Xi'_t't~-" },
      {       8614, "Xi*_t't+" },
      {      -8614, "Xi*_t't~-" },
      {       8622, "Xi'_t't++" },
      {      -8622, "Xi'_t't~--" },
      {       8624, "Xi*_t't++" },
      {      -8624, "Xi*_t't~--" },
      {       8632, "Omega'_t't+" },
      {      -8632, "Omega'_t't~-" },
      {       8634, "Omega*_t't+" },
      {      -8634, "Omega*_t't~-" },
      {       8642, "Omega'_t'tc++" },
      {      -8642, "Omega'_t'tc~--" },
      {       8644, "Omega*_t'tc++" },
      {      -8644, "Omega*_t'tc~--" },
      {       8652, "Omega'_t'tb+" },
      {      -8652, "Omega'_t'tb~-" },
      {       8654, "Omega*_t'tb+" },
      {      -8654, "Omega*_t'tb~-" },
      {       8662, "Omega'_t'tt++" },
      {      -8662, "Omega'_t'tt~--" },
      {       8664, "Omega*_t'tt++" },
      {      -8664, "Omega*_t'tt~--" },
      {       8672, "Omega_t'b't +" },
      {      -8672, "Omega_t'b't ~-" },
      {       8712, "Xi'_t'b'0" },
      {      -8712, "Xi'_t'b'~0" },
      {       8714, "Xi*_t'b'0" },
      {      -8714, "Xi*_t'b'~0" },
      {       8722, "Xi'_t'b'+" },
      {      -8722, "Xi'_t'b'~-" },
      {       8724, "Xi*_t'b'+" },
      {      -8724, "Xi*_t'b'~-" },
      {       8732, "Omega'_t'b'0" },
      {      -8732, "Omega'_t'b'~0" },
      {       8734, "Omega*_t'b'0" },
      {      -8734, "Omega*_t'b'~0" },
      {       8742, "Omega'_t'b'c+" },
      {      -8742, "Omega'_t'b'c~-" },
      {       8744, "Omega*_t'b'c+" },
      {      -8744, "Omega*_t'b'c~-" },
      {       8752, "Omega'_t'b'b0" },
      {      -8752, "Omega'_t'b'b~0" },
      {       8754, "Omega*_t'b'b0" },
      {      -8754, "Omega*_t'b'b~0" },
      {       8762, "Omega'_t'b't+" },
      {      -8762, "Omega'_t'b't~-" },
      {       8764, "Omega*_t'b't+" },
      {      -8764, "Omega*_t'b't~-" },
      {       8772, "Omega'_t'b'b'0" },
      {      -8772, "Omega'_t'b'b'~0" },
      {       8774, "Omega*_t'b'b'0" },
      {      -8774, "Omega*_t'b'b'~0" },
      {       8812, "Xi'_t't'+" },
      {      -8812, "Xi'_t't'~-" },
      {       8814, "Xi*_t't'+" },
      {      -8814, "Xi*_t't'~-" },
      {       8822, "Xi'_t't'++" },
      {      -8822, "Xi'_t't'~--" },
      {       8824, "Xi*_t't'++" },
      {      -8824, "Xi*_t't'~--" },
      {       8832, "Omega'_t't'+" },
      {      -8832, "Omega'_t't'~-" },
      {       8834, "Omega*_t't'+" },
      {      -8834, "Omega*_t't'~-" },
      {       8842, "Omega'_t't'c++" },
      {      -8842, "Omega'_t't'c~--" },
      {       8844, "Omega*_t't'c++" },
      {      -8844, "Omega*_t't'c~--" },
      {       8852, "Omega'_t't'b+" },
      {      -8852, "Omega'_t't'b~-" },
      {       8854, "Omega*_t't'b+" },
      {      -8854, "Omega*_t't'b~-" },
      {       8862, "Omega'_t't't++" },
      {      -8862, "Omega'_t't't~--" },
      {       8864, "Omega*_t't't++" },
      {      -8864, "Omega*_t't't~--" },
      {       8872, "Omega'_t't'b'+" },
      {      -8872, "Omega'_t't'b'~-" },
      {       8874, "Omega*_t't'b'+" },
      {      -8874, "Omega*_t't'b'~-" },
      {       8884, "Omega*_t't't'++" },
      {      -8884, "Omega*_t't't'~--" },
      {    9221132, "Theta+" },
      {    9331122, "Phi--" },
      { 1001001002, "Hydrogen" },
      { 1002001002, "Deuteron" },
      { 1003001002, "Tritium" },
      { 1003002001, "He3" },
      { 1004002001, "Alpha (He4)" }
  };

  int lnames = sizeof(SNames)/sizeof(SNames[0]);
  for( int k=0; k!=lnames; ++k) {
      m.insert( std::make_pair( SNames[k].pid, std::string(SNames[k].pname)) );
  }

  return m;
}  // ParticleNameInit()

void writeParticleNameLine( int i, std::ostream & os  )
{
    if ( validParticleName( i ) ) {
        std::string pn = particleName( i );
	os << "  PDT number: " ;
	os.width(12);
	os << i << "  PDT name: " << pn << std::endl;

    }
    return;
}  // writeParticleNameLine()

} // unnamed namespace

//
// getParticleNameMap is the ONLY function allowed to call ParticleNameInit
//
ParticleNameMap const &  getParticleNameMap()
{
  static  ParticleNameMap const &  pmap = ParticleNameInit();
  return pmap;
}  // getParticleNameMap()

bool validParticleName( const int pid )
{
    static  ParticleNameMap const &  pmap = getParticleNameMap();

    ParticleNameMap::const_iterator const cit = pmap.find( pid );
    return ( cit == pmap.end() )
         ? false
	 : true;
}  // validParticleName()

std::string  particleName( const int pid )
{
    static  ParticleNameMap const &  pmap = getParticleNameMap();

    ParticleNameMap::const_iterator const cit = pmap.find( pid );
    return ( cit == pmap.end() )
         ? std::string("not defined")
	 : cit->second;
}  // particleName()

//
// list all the defined names
//
void  listHepPDTParticleNames( std::ostream & os  )
{
    os << "     HepPDT Particle List" << std::endl;
    os << std::endl;

    // simple: static  ParticleNameMap const &  pmap = getParticleNameMap();
    // simple: for( ParticleNameMap::const_iterator cit = pmap.begin(), mend = pmap.end(); 
    // simple:                                 cit != mend;
	// simple: 			  ++cit ) {
	// simple: os << "  PDT number: " ;
	// simple: os.width(12);
	// simple: os << cit->first << "  PDT name: " << cit->second << std::endl;
    // simple: }
    int id, i, j, q1, q2, q3, l, m, n;
    // special cases
    for( id=1; id<101; ++id) {
        writeParticleNameLine(  id, os );
        writeParticleNameLine( -id, os );
    }
    for( i=11; i<1000; ++i) {
        id = i*10;
        writeParticleNameLine(  id, os );
        writeParticleNameLine( -id, os );
    }
    // SUSY, technicolor, etc.
    for( n=1; n<6; ++n) {
        for( q2=0; q2<10; ++q2) {
	    for( q1=0; q1<10; ++q1) {
	        for( j=0; j<10; ++j) {
		    for( m=0; m<10; ++m) {
		        for( l=0; l<7; ++l) {
			    id = 1000000*n+100000*m+10000*l+100*q2+10*q1+j;
        		    writeParticleNameLine(  id, os );
        		    writeParticleNameLine( -id, os );
			}
		    }
		}
	    }
	}
    }
    // miscellaneous generator particles
    for( i=1; i<100; ++i) {
	id = 9900000+i;
        writeParticleNameLine(  id, os );
        writeParticleNameLine( -id, os );
    }
    for( l=1; l<9; ++l) {
        for( q3=0; q3<10; ++q3) {
            for( q2=0; q2<10; ++q2) {
		for( q1=0; q1<10; ++q1) {
	            for( j=0; j<10; ++j) {
			id = 9900000+10000*l+1000*q3+100*q2+10*q1+j;
        		writeParticleNameLine(  id, os );
        		writeParticleNameLine( -id, os );
		    }
		}
	    }
	}
    }
    // diquark
    for( i=11; i<100; ++i) {
        for( j=0; j<10; ++j) {
            id = 100*i+j;
            writeParticleNameLine(  id, os );
            writeParticleNameLine( -id, os );
	}
    }
    // mesons
    for( q2=1; q2<10; ++q2) {
	for( q1=1; q1<10; ++q1) {
	    for( j=1; j<10; ++j) {
		for( m=0; m<9; ++m) {
		    for( l=0; l<10; ++l) {
			id = 100000*m+10000*l+100*q2+10*q1+j;
        		writeParticleNameLine(  id, os );
        		writeParticleNameLine( -id, os );
			id = 9000000+100000*m+10000*l+100*q2+10*q1+j;
        		writeParticleNameLine(  id, os );
        		writeParticleNameLine( -id, os );
		    }
		}
	    }
	}
    }
    // baryons
    for( q3=1; q3<10; ++q3) {
	for( q2=1; q2<10; ++q2) {
	    for( q1=1; q1<10; ++q1) {
		for( j=1; j<10; ++j) {
		    id = 1000*q3+100*q2+10*q1+j;
        	    writeParticleNameLine(  id, os );
        	    writeParticleNameLine( -id, os );
		}
	    }
	}
    }
    // pentaquarks
    for( l=1; l<9; ++l ) {
        for ( m=1; m<9; ++m ) {
	    for( q3=1; q3<9; ++q3) {
		for( q2=1; q2<9; ++q2) {
		    for( q1=1; q1<9; ++q1) {
			id = 9*1000000+l*100000+m*10000+1000*q3+100*q2+10*q1+2;
        		writeParticleNameLine(  id, os );
        		writeParticleNameLine( -id, os );
		    }
		}
	    }
        }
    }
    // ions
    for( i=1; i<3; ++i) {
	for( m=1; m<5; ++m) {
            for( j=1; j<3; ++j) {
		id = 1000000000+1000000*m+1000*i+j;
        	writeParticleNameLine(  id, os );
        	writeParticleNameLine( -id, os );
	    }
	}
    }
    return;
}  // listHepPDTParticleNames()

}	// HepPDT
